@@ -25,4 +25,12 @@ if not global then
     global  = _G
 end
 
-LUATEXVERSION = status.luatex_version/100 + tonumber(status.luatex_revision)/1000
+LUATEXVERSION = status.luatex_version/100
+              + tonumber(status.luatex_revision)/1000
+
+LUATEXENGINE  = status.luatex_engine and string.lower(status.luatex_engine)
+             or (string.find(status.banner,"LuajitTeX") and "luajittex" or "luatex")
+
+JITSUPPORTED  = LUATEXENGINE == "luajittex" or jit
+
+INITEXMODE    = status.ini_version
